/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust file filters (GCFileFilter)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)GCFileFilter.java    0.95 2001/09/01
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.io.File;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javax.swing.filechooser.FileFilter;


/**
 * GCFileFilter - Abstract application-specific file filter
 *
 * @version 0.98 2001/02/12
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
abstract public class GCFileFilter extends FileFilter
    implements GCFileFilterKeys
{
    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension(s).
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    abstract public boolean isMatchingExtension(String extension);

    // =========================================================================
    // IMPLEMENTS FILEFILTER
    // =========================================================================

    /**
     * Tests whether or not the specified abstract pathname should be
     * included in a pathname list.
     *
     * @param file  abstract pathname to be tested
     * @return      true if pathname should be included; otherwise, false
     */
    public boolean accept(File file)
    {
        if (file.isDirectory() == true)
            return true;

        String extension = getExtension(file);

        if (extension != null)
            return isMatchingExtension(extension);

        return false;
    }

    /**
     * Returns the file filter's description
     *
     * @return  string specifying description of file filter
     */
    public String getDescription()
    {
        return description;
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Sets the file filter's description
     *
     * @param description  string specifying description of file filter
     */
    protected void setDescription(String description)
    {
        this.description = description;
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Returns the abstract pathname's file extension. If no extension is
     * found, null is returned. Assumes a Un*x style pathname.
     *
     * @param file  abstract pathname to be tested
     * @return      string specifying file extension (without dot)
     *              of abstract pathname
     */
    private String getExtension(File file)
    {
        String fileName = file.getName();
        int dotIndex = fileName.lastIndexOf('.');
        String extension = null;

        if ((dotIndex > 0) && (dotIndex < fileName.length() - 1))
            extension = fileName.substring(dotIndex+1).toLowerCase();

        return extension;
    }

    // =========================================================================
    // CLASS INTERFACE (Package)
    // =========================================================================

    /**
     * Formats messages for display.
     *
     * @param propertyKey  string containing property key for format string
     * @param arg          object for single argument of format string
     * @return             string containing formatted message
     */
    static String formatMessage(String propertyKey, Object arg)
    {
        String formatStr = getBundleString(propertyKey);
        Object[] args = { arg };
        MessageFormat mf = new MessageFormat(formatStr);

        return mf.format(args);
    }

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        String value = null;

        try
        {
            value = bundle.getString(propertyKey);
        }
        catch (MissingResourceException mre)
        {
            System.err.println(mre);
        }

        if (value == null)
            value = "Could not find resource: " + propertyKey + "  ";

        return value;
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;

    static
    {
        final String propertyFile = GCFileFilter.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private String description;
}


/**
 * ClassificationFileFilter - Filters out files not pertaining to supervised
 *                            shaving in the GC application.
 *
 * @version 0.99 2001/09/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class ClassificationFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public ClassificationFileFilter()
    {
        setDescription(formatMessage(CLASSIFICATION_FILE_FILTER_DESCRIPTION,
                                     classificationExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(classificationExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String classificationExtension = "clf";
}


/**
 * ConfigFileFilter - Filters out files not containing GC settings.
 *
 * @version 0.98 2001/02/12
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class ConfigFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public ConfigFileFilter()
    {
        setDescription(formatMessage(CONFIG_FILE_FILTER_DESCRIPTION,
                                     configExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(configExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String configExtension = "cf";
}


/**
 * AsciiFileFilter - Filters out files not containing GC data.
 *
 * @version 0.99 2001/05/24
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class AsciiFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public AsciiFileFilter()
    {
        setDescription(formatMessage(ASCII_FILE_FILTER_DESCRIPTION,
                                     asciiExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return (extension.equals(asciiExtension));
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String asciiExtension = "txt";
}


/**
 * TabSeparatedValuesFileFilter - Filters out files not containing GC data.
 *
 * @version 1.00 2001/09/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class TabSeparatedValuesFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public TabSeparatedValuesFileFilter()
    {
        setDescription(formatMessage(TSV_FILE_FILTER_DESCRIPTION,
                                     tsvExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(tsvExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String tsvExtension = "tsv";
}


/**
 * CommaSeparatedValuesFileFilter - Filters out files not containing GC data.
 *
 * @version 1.00 2001/09/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class CommaSeparatedValuesFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public CommaSeparatedValuesFileFilter()
    {
        setDescription(formatMessage(CSV_FILE_FILTER_DESCRIPTION,
                                     csvExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(csvExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String csvExtension = "csv";
}


/**
 * FormattedAsciiFileFilter - Filters out files not containing GC data.
 *
 * @version 0.98 2001/02/16
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class FormattedAsciiFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public FormattedAsciiFileFilter()
    {
        setDescription(formatMessage(FORMATTED_ASCII_FILE_FILTER_DESCRIPTION,
                                     formattedAsciiExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(formattedAsciiExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String formattedAsciiExtension = "fix";
}


/**
 * MicrosoftExcelFileFilter - Filters out files not containing GC data.
 *
 * @version 0.98 2001/02/16
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class MicrosoftExcelFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public MicrosoftExcelFileFilter()
    {
        setDescription(formatMessage(MICROSOFT_EXCEL_FILE_FILTER_DESCRIPTION,
                                     msExcelExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(msExcelExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String msExcelExtension = "xls";
}


/**
 * SasFileFilter - Filters out files not containing GC data.
 *
 * @version 0.98 2001/02/16
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class SasFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public SasFileFilter()
    {
        setDescription(formatMessage(SAS_FILE_FILTER_DESCRIPTION,
                                     sasExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(sasExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String sasExtension = "sd2";
}


/**
 * MatlabFileFilter - Filters out files not containing GC data.
 *
 * @version 0.98 2001/02/16
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class MatlabFileFilter extends GCFileFilter
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public MatlabFileFilter()
    {
        setDescription(formatMessage(MATLAB_FILE_FILTER_DESCRIPTION,
                                     matlabExtension));
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Tests whether or not the abstract pathname's file extension matches
     * this filter's expected extension.
     *
     * @param extension  file extension to be tested
     * @return           true if extension matches; otherwise, false
     */
    public boolean isMatchingExtension(String extension)
    {
        return extension.equals(matlabExtension);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String matlabExtension = "mat";
}

