
package org.mdanderson.biostat.geneclust;

import java.io.BufferedWriter;
import java.io.IOException;


/**
 * SProcess - Abstract data passing between the user interface and Sinterpreter.
 *
 * @version 0.95 2001/02/20
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
abstract public class SProcess
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================
    /**
     * @param dialog                  dialog allows monitoring of sim progress
     * @throws NullPointerException if dialog is null
     */
    public SProcess(SimulationDialog dialog)
    {
        if (dialog == null)
            throw new NullPointerException("dialog");
        this.dialog = dialog;
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Returns reference to dialog where simulation progress can be monitored.
     *
     * @return  simulation dialog object
     */
    public SimulationDialog getDialog()
    {
        return dialog;
    }

    /**
     * Runs the simulation, managing the S interpreter processing occuring in external
     * process.
     *
     * @throws Exception if anything goes wrong
     */
    public void run(String param)
      throws Exception
    {
        sinterpreter = new Sinterpreter(this);

        sinterpreter.runSimulation();
        dialog.displayAndWait();
        if (dialog.getProcessing())
            sinterpreter.interrupt();
        sinterpreter.endSimulation();
    }

    /**
     * Writes all S interpreter input to the output stream.
     *
     * @param bw  output stream writer
     */
    public abstract void writeSinterpreterInput(SBufferedWriter bw);

    /**
     * Writes the S interpreter input required to interrupt the simulation to the
     * output stream. The interrupt will only happen after the "instruction"
     * S interpreter is currently executing completes.
     *
     * @param bw  output stream writer
     */
    public void writeSinterpreterInterrupt(SBufferedWriter bw)
    {
        try
        {
            synchronized(bw)
            {
                bw.write(3);    /* ETX a.k.a. Ctrl-C */
                bw.newLine();
                bw.write("stop('***Simulation cancelled!')");
                bw.newLine();

                bw.flush();
            }
        }
        catch (IOException ioe)
        {
            System.err.println(ioe);
        }
    }

    /**
     * Writes the S interpreter input required to close the graphics device and
     * terminate the external process.
     *
     * @param bw  output stream writer
     */
    public void writeSinterpreterTermination(SBufferedWriter bw)
    {
        try
        {
            synchronized(bw)
            {
                bw.write("dev.off()");
                bw.newLine();
                bw.write("q()");
                bw.newLine();

                bw.flush();
            }
        }
        catch (IOException ioe)
        {
            System.err.println(ioe);
        }
        finally
        {
            try
            {
                bw.close();
            }
            catch (IOException ioe)
            {
                System.err.println(ioe);
            }
            bw = null;
        }
    }

    private Sinterpreter sinterpreter;
    protected SimulationDialog dialog;
}

