/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for generic stream monitoring class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)DefaultStreamMonitor.java    1.0 2001/02/01
 *
 * Copyright (c) 2000 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ResourceBundle;


/**
 * DefaultStreamMonitor - Generic application stream monitoring
 *
 * @version 1.0 2001/02/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class DefaultStreamMonitor
  implements Runnable, DefaultStreamMonitorKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying input stream, and output stream
     *
     * @param is        input stream
     * @param redirect  output stream
     */
    public DefaultStreamMonitor(InputStream is, OutputStream redirect)
    {
        this.is = is;
        this.os = redirect;
        this.echo = false;
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Disable echo of processed input from external process to 'stdout' stream.
     */
    public void disableEcho()
    {
        echo = false;
    }

    /**
     * Enable echo of processed input from external process to 'stdout' stream.
     */
    public void enableEcho()
    {
        echo = true;
    }

    // =========================================================================
    // IMPLEMENTS RUNNABLE
    // =========================================================================

    /**
     * Thread entry-point for DefaultStreamMonitor object.
     * Handles monitoring the input stream, processing what it reads and
     * writing it to the output stream.
     */
    public void run()
    {
        System.err.println(getBundleString(STREAMMONITOR_STARTED));
        PrintWriter pw = null;

        if (os != null)
            pw = new PrintWriter(os);

        InputStreamReader isr = new InputStreamReader(is);
        BufferedReader br = new BufferedReader(isr);

        System.err.println(getBundleString(STREAMMONITOR_AWAITING_INPUT));
        readInput(br, pw);
        System.err.println(getBundleString(STREAMMONITOR_DONE));
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Echoes processed input from external process to 'stdout' stream.
     * All echoed output is prefixed by tag to annotate its origin.
     *
     * @param line  string containing reformatted external process output
     */
    protected void echoOutput(String line)
    {
        System.out.println(TAG + line);
    }

    /**
     * Postprocesses the output from the input stream. This implementation
     * simply returns its argument. 
     *
     * @param line  string containing external process output
     * @return      string containing possibly reformatted output
     */
    protected String processLine(String line)
    {
        return line;
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Reads from the input stream, processes what it reads and
     * writes it to the output stream.
     *
     * @param br  input stream reader
     * @param pw  output stream writer
     */
    private void readInput(BufferedReader br, PrintWriter pw)
    {
        try
        {
            String line = null;
            while ((line = br.readLine()) != null)
            {
                line = processLine(line);

                if (pw != null)
                    pw.println(line);

                if (echo == true)   
                    echoOutput(line);
            }
        }
        catch (IOException ioe)
        {
            System.err.println(getBundleString(STREAMMONITOR_END_OF_INPUT));
            // No more input
        }
        finally
        {
            try
            {
                br.close();
            }
            catch (IOException ioe)
            {
                System.err.println(ioe);
            }
            if (pw != null)
                pw.close();
        }
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;
    private static String TAG;

    static
    {
        final String propertyFile = DefaultStreamMonitor.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);

        TAG = getBundleString(STREAMMONITOR_TAG);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    InputStream is;
    OutputStream os;
    volatile boolean echo;
}

