/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for webpage link simulation class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)UrlLabel.java    1.0 2001/08/25
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.util.EventListener;
import javax.swing.JTextField;
import javax.swing.event.EventListenerList;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;


/**
 * UrlLabel - Simulates a link on a webpage.
 *
 * @version 1.0 2001/08/07
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class UrlLabel extends JTextField
  implements HyperlinkSource
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor specifying text and url
     *
     * @param text  string containing text to be displayed by the label
     * @param url   URL to invoke when activated
     */
    public UrlLabel(String text, URL url)
    {
        super(text);
        setEditable(false);
        setBorder(null);
        setOpaque(true);
        setForeground(unvisitedColor);
        this.url = url;
        this.listenerList = new EventListenerList();
        addMouseListener(new MouseEventHandler());
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Invoked by Swing to draw components. Augments the parent's method to
     * draw a line beneath the label text to give the appearance of a
     * web page link.
     *
     * @param g  <code>Graphics</code> context in which to paint
     */
    public void paint(Graphics g)
    {
        super.paint(g);
        Rectangle r = g.getClipBounds();
        FontMetrics fm = this.getFontMetrics(this.getFont());

        int x1 = 0;
        int x2 = fm.stringWidth(this.getText());
        int y = r.height - fm.getDescent();

        g.drawLine(x1, y, x2, y);
    }

    /**
     * Sets the foreground color of an unvisited link
     *
     * @param color  color used as label foreground color
     */ 
    public void setUnvisitedColor(Color color)
    {
        unvisitedColor = color;
    }

    /**
     * Sets the foreground color of a (previously) visited link
     *
     * @param color  color used as label foreground color
     */ 
    public void setVisitedColor(Color color)
    {
        visitedColor = color;
    }

    // =========================================================================
    // IMPLEMENTS HYPERLINKSOURCE
    // =========================================================================

    /**
     * Adds a hyperlink listener for notification of any changes,
     * for example when a link is selected and entered.
     *
     * @param listener  hyperlink listener
     */
    public void addHyperlinkListener(HyperlinkListener listener)
    {
        listenerList.add(HyperlinkListener.class, listener);
    }

    /**
     * Removes a hyperlink listener.
     *
     * @param listener  hyperlink listener
     */
    public void removeHyperlinkListener(HyperlinkListener listener)
    {
        listenerList.remove(HyperlinkListener.class, listener);
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Handles Hyperlink ENTERED events. This implementation changes
     * the cursor to a hand and causes listener notification to occur.
     */
    protected void handleEntered()
    {
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        fireHyperlinkUpdate(HyperlinkEvent.EventType.ENTERED);
    }

    /**
     * Handles Hyperlink EXITED events. This implementation changes
     * the cursor back to normal and causes listener notification to occur.
     */
    protected void handleExited()
    {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        fireHyperlinkUpdate(HyperlinkEvent.EventType.EXITED);
    }

    /**
     * Handles Hyperlink ACTIVATED events. This implementation changes
     * the color of the URL label and causes listener notification to occur.
     */
    protected void handleActivated()
    {
        setForeground(visitedColor);
        fireHyperlinkUpdate(HyperlinkEvent.EventType.ACTIVATED);
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Notifies all listeners that have registered interest for notification
     * on this event type. The listener list is processed last to first.
     *
     * @param et  <code>HyperlinkEvent.EventType</code> event type
     * @see javax.swing.event.HyperlinkEvent.EventType#ENTERED
     * @see javax.swing.event.HyperlinkEvent.EventType#EXITED
     * @see javax.swing.event.HyperlinkEvent.EventType#ACTIVATED
     */
    private void fireHyperlinkUpdate(HyperlinkEvent.EventType et)
    {
        EventListener[] listeners;
        HyperlinkEvent event = null;

        // Process the listeners last to first
        listeners = listenerList.getListeners(HyperlinkListener.class);
        for (int i = listeners.length-1; i >= 0; i--)
        {
            HyperlinkListener hyperlinkListener;

            if (event == null)
                event = new HyperlinkEvent(this, et, url);

            hyperlinkListener = (HyperlinkListener)listeners[i];
            hyperlinkListener.hyperlinkUpdate(event);
        }
    }

    // =========================================================================
    // INNER CLASSES
    // =========================================================================

    /**
     * MouseEventHandler - Responds to mouse events.
     *
     * @version 1.0 2001/08/07
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class MouseEventHandler extends MouseAdapter
    {
        // =====================================================================
        // IMPLEMENTS MOUSELISTENER
        // =====================================================================

        /**
         * Invoked when the mouse enters a component.
         *
         * @param event  event which indicates that a mouse action occurred
         */
        public void mouseEntered(MouseEvent event)
        {
            handleEntered();
        }

        /**
         * Invoked when the mouse exits a component.
         *
         * @param event  event which indicates that a mouse action occurred
         */
        public void mouseExited(MouseEvent event)
        {
            handleExited();
        }

        /**
         * Invoked when the mouse has been clicked on a component.
         *
         * @param event  event which indicates that a mouse action occurred
         */
        public void mouseClicked(MouseEvent event)
        {
            handleActivated();
        }
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final Color purple;

    static
    {
        {
            //final int rgbPurple [] = { 160, 32, 240 };
            final int rgbPurple [] = { 82, 24, 140 };
            purple = new Color(rgbPurple [0],
                               rgbPurple [1],
                               rgbPurple [2]);
        }
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private transient EventListenerList listenerList;
    private URL url;
    private Color unvisitedColor = Color.blue;
    private Color visitedColor = purple;
}

