/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for linked list routines
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     ANSI C
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  geneclust
 *
 *============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "GeneClust.h"
#include "LinkedList.h"


/*
 * Private Function Prototypes
 */
static Node_t *ListFindElement (const Node_t *list, ELEMENT element);
static Node_t *NodeCreate (ELEMENT element, unsigned position);
static void NodeDestroy (Node_t *node);
static void NodeInit (Node_t *node, ELEMENT element, unsigned position);
static void NodePrint (const Node_t *node);


/*
 * Public Routines
 */

/*============================================================================
 *
 * Name:: ListInit
 *
 * Description::
 *     Initializes the list head to NULL
 *
 * Inputs::
 *     list - Handle to list head to be initialized
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListInit (Node_t **list)
{
    assert (list != NULL);

    *list = NULL;
}


/*============================================================================
 *
 * Name:: ListPrepend
 *
 * Description::
 *     Prepends a new node to the list containing the desired values if no
 *     node already within the list contains the 'element' value. If a node
 *     does exist with this value, its internal list is updated to include
 *     the 'position' parameter. 
 *
 * Inputs::
 *     list     - Handle to list head where node would be created.
 *     element  - Data value of node (used as exclusive index key).
 *     position - Line number within the file where the data value was read.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListPrepend (Node_t **list, ELEMENT element, unsigned position)
{
    Node_t *head;

    assert (list != NULL);

    head = *list;
    ListInsert (list, head, element, position);
}


/*============================================================================
 *
 * Name:: ListAppend
 *
 * Description::
 *     Appends a new node to the list containing the desired values if no
 *     node already within the list contains the 'element' value. If a node
 *     does exist with this value, its internal list is updated to include
 *     the 'position' parameter.
 *
 * Inputs::
 *     list     - Handle to list head where node would be created.
 *     element  - Data value of node (used as exclusive index key).
 *     position - Line number within the file where the data value was read.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListAppend (Node_t **list, ELEMENT element, unsigned position)
{
    Node_t *tail;

    assert (list != NULL);

    tail = *list;
    for (; tail && tail->next; tail = tail->next);
    ListInsert (list, tail, element, position);
}


/*============================================================================
 *
 * Name:: ListInsert
 *
 * Description::
 *     Inserts a new node to the list containing the desired values if no
 *     node already within the list contains the 'element' value. If a node
 *     does exist with this value, its internal list is updated to include
 *     the 'position' parameter.
 *
 * Inputs::
 *     list     - Handle to list head where node would be created.
 *     q        - Pointer to node preceding the location within the list
 *                where the new node would be created.
 *     element  - Data value of node (used as exclusive index key).
 *     position - Line number within the file where the data value was read.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListInsert (Node_t **list, Node_t *q, ELEMENT element, unsigned position)
{
    Node_t *head;
    Node_t *node;

    assert (list != NULL);

    head = *list;
    if ((node = ListFindElement (head, element)) == NULL)
         node = NodeCreate (element, position);
    else
    {
         unsigned *tp;

         tp = realloc (node-> positions,
                       (node->numPositions+1)*sizeof(unsigned));
         if (tp == NULL)
         {
             ErrMessage_t errMessage;

             sprintf (errMessage, "Error inserting into list: %s\n",
                      strerror (errno));
             Abend (errMessage);
         }

         node->numPositions++;
         node->positions = tp;
         node->positions[(node->numPositions)-1] = position;

         return;
    }

    if (head)
    {
        node->next = q->next;
        q->next = node;
    }
    else
    {
        node->next = NULL;
        *list = node;
    }
}


/*============================================================================
 *
 * Name:: ListPrint
 *
 * Description::
 *     Prints the contents of each node in the list.
 *
 * Inputs::
 *     list - Pointer to node where printing should begin.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListPrint (const Node_t *list)
{
    Node_t *node;
    int id = 0;

    assert (list != NULL);

    for (node = (Node_t *)list; node != NULL; node = node->next)
    {
        id++;
        printf ("list element %d = %p\n", id, node);
        NodePrint (node);
    }
}


/*============================================================================
 *
 * Name:: ListLength
 *
 * Description::
 *     Counts how many nodes are in the list.
 *
 * Inputs::
 *     list - Pointer to node where counting should begin.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     Returns the number of nodes in the list.
 *
 *============================================================================*/

size_t ListLength (const Node_t *list)
{
    Node_t *node;
    size_t length = 0;

    assert (list != NULL);

    for (node = (Node_t *)list; node != NULL; node = node->next)
    {
        length++;
    }

    return length;
}


/*============================================================================
 *
 * Name:: ListDestroy
 *
 * Description::
 *     Destroys the contents of the list, freeing any allocated memory.
 *
 * Inputs::
 *     list - Handle to list head to be destroyed
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

void ListDestroy (Node_t **list)
{
    Node_t *tail;

    /* Termination condition */
    if (list == NULL)
        return;

    /* Recursively descend to end of list */
    {
        Node_t *head;

        head = tail = *list;
        if (head->next != NULL)
            ListDestroy (&(head->next));
    }

    /* Free allocated memory */
    NodeDestroy (tail);
    free (tail);
    tail = NULL;
}


/*
 * Private Routines
 */

/*============================================================================
 *
 * Name:: ListFindElement
 *
 * Description::
 *     Searches the nodes of the list for one containing 'element' value. 
 *
 * Inputs::
 *     list    - Pointer to node where counting should begin.
 *     element - Data value of node (used as exclusive index key).
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     Returns pointer to node if match found; otherwise, NULL.
 *
 *============================================================================*/

static Node_t *ListFindElement (const Node_t *list, ELEMENT element)
{
    Node_t *node;

    for (node = (Node_t *)list; node != NULL; node = node->next)
    {
        if (node->element == element)
            break;
    }

    return node;
}


/*============================================================================
 *
 * Name:: NodeCreate
 *
 * Description::
 *     Creates a new Node_t data structure and initializes it.
 *
 * Inputs::
 *     (None)
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     Returns pointer to newly created instance of Node_t.
 *
 *============================================================================*/

static Node_t *NodeCreate (ELEMENT element, unsigned position)
{
    Node_t *node;

    node = (Node_t *) malloc (sizeof(Node_t));
    if (node == NULL)
    {
         ErrMessage_t errMessage;

         sprintf (errMessage, "Error creating list node: %s\n",
                  strerror (errno));
         Abend (errMessage);
    }

    NodeInit (node, element, position);

    return node;
}


/*============================================================================
 *
 * Name:: NodeDestroy
 *
 * Description::
 *     Destroys the contents of the node, freeing any allocated memory.
 *
 * Inputs::
 *     node - Pointer to node to be destroyed
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

static void NodeDestroy (Node_t *node)
{
    assert (node != NULL);

#if 0 && defined(DEBUG)
    fprintf (stderr, "NodeDestroy(node = 0x%p)\n", node);
#endif

    if (node->positions != NULL) {
        free (node->positions);
        node->positions = NULL;
    }
#ifdef DEBUG
    memset (node, '\0', sizeof (Node_t));
#endif
}


/*============================================================================
 *
 * Name:: NodeInit
 *
 * Description::
 *     Initializes a Node_t data structure.
 *
 * Inputs::
 *     node - Pointer to Node_t to be initialized.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

static void NodeInit (Node_t *node, ELEMENT element, unsigned position)
{
    assert (node != NULL);

    node->element = element;
    node->positions = (unsigned *) malloc (sizeof(unsigned));
    if (node->positions == NULL)
    {
         ErrMessage_t errMessage;

         sprintf (errMessage, "Error creating list node item: %s\n",
                  strerror (errno));
         Abend (errMessage);
    }

    node->positions[0] = position;
    node->numPositions = 1;
    node->next = NULL;
}


/*============================================================================
 *
 * Name:: NodePrint
 *
 * Description::
 *     Prints the contents of a node.
 *
 * Inputs::
 *     node - Pointer to Node_t to be printed.
 *
 * Outputs::
 *     (None)
 *
 * Return Value::
 *     (None)
 *
 *============================================================================*/

static void NodePrint (const Node_t *node)
{
    register unsigned i;

    assert (node != NULL);

    printf ("\tnode->element = %d\n", node->element);
    printf ("\tnode->numPositions = %u\n", node->numPositions);
    printf ("\tnode->positions\n");
    for (i = 0; i < node->numPositions; i++)
        printf ("\t\t[%d] = %u \n", i, node->positions[i]);
}

